#include "STC15Wxx.h"
#include "intrins.h"
#include "I2C.h"
// #include "temp.h"
#include "EEPROM.h"
#include "delay.h"

typedef unsigned char uchar;
typedef unsigned int uint;
typedef unsigned long ulong;

// *************************************
// CPU参数相关
// *************************************

#define FOSC 22118400L              //CPU主频
#define T1MS (65536-FOSC/1000)      //1T模式下定时器溢出时间

#define ADC_POWER      0x80            // ADC电源控制位
#define ADC_FLAG       0x10            // ADC完成标志
#define ADC_START      0x08            // ADC起始控制位
#define ADC_SPEED      0x00            // ADC速度180个时钟

// *************************************
// 固定设置
// *************************************

#define TIME_SET_TIMEOUT       20000      // 设置超时时长
#define TEMP_MAX_SHOW_TIME     30         // 温度最长显示时长
#define TEMP_MAX_HIDE_TIME     60         // 温度最长隐藏时长
#define MAX_BRIGHTNESS         10         // 强光值
#define LIGHT_WEAK_VALUE       550        // 弱光值
#define ROTATE_DELAY           1000       // 旋转延迟时间

// *************************************
// 常量
// *************************************

// 当前显示内容设置
#define MODE_SHOW_TIME          1    // 显示时间
#define MODE_SHOW_SECONDS       2    // 显示秒
#define MODE_SET_HOUR           3    // 设置小时
#define MODE_SET_MINUTE         4    // 设置分钟
#define MODE_SHOW_TEMPERTURE    5    // 显示温度
#define MODE_SET_ALARM_ENABLED  6    // 设置闹铃小时
#define MODE_SET_ALARM_HOUR     7    // 设置闹铃小时
#define MODE_SET_ALARM_MINUTE   8    // 设置闹铃分钟
#define MODE_SET_TEMP_SHOW      9    // 设置温度显示时间
#define MODE_SET_TEMP_NO_SHOW   10   // 设置温度隐藏时间
#define MODE_SET_ROT_ENABLED    11   // 设置是否整点报时
#define MODE_SET_ROT_START      12   // 设置整点报时启动时间
#define MODE_SET_ROT_END        13   // 设置整点报时停止时间
#define MODE_SET_BRIGHTNESS     14   // 设置整点报时停止时间

#define ROTATION_MODE_AUTO      0    // 屏幕旋转模式：自动
#define ROTATION_MODE_NORMAL    1    // 屏幕旋转模式：正常
#define ROTATION_MODE_REVERSE   2    // 屏幕旋转模式：颠倒

// EEPROM相关
#define CMD_IDLE          0          // 空闲模式
#define CMD_READ          1          // IAP字节读命令
#define CMD_PROGRAM       2          // IAP字节编程命令
#define CMD_ERASE         3          // IAP扇区擦除命令

#define EEPROM_VERSION    1          // EEPROM版本

#define EEP_ADDR_SECTION              0x0000    // 存储扇区开始地址
#define EEP_ADDR_VERSION              0x0001    // 存储扇区开始地址
#define EEP_ADDR_TEMP_VISIBLE         0x0002    // 显示温度时长
#define EEP_ADDR_TEMP_INVISIBLE       0x0003    // 温度隐藏时长
#define EEP_ADDR_IS_RING_ON_TIME      0x0004    // 是否整点报时
#define EEP_ADDR_RING_ON_TIME_START   0x0005    // 整点报时开始时间
#define EEP_ADDR_RING_ON_TIME_STOP    0x0006    // 整点报时结束时间
#define EEP_ADDR_ALARM_ENABLE         0x0007    // 是否开启闹钟
#define EEP_ADDR_ALARM_HOUR           0x0008    // 闹钟时
#define EEP_ADDR_ALARM_MINUTE         0x0009    // 闹钟分
#define EEP_ADDR_BRIGHTNESS           0x000A    // 亮度
#define EEP_ADDR_STRONG_LIGHT_START   0x000B    // 强光开始时间
#define EEP_ADDR_STRONG_LIGHT_END     0x000C    // 强光关闭时间
#define EEP_ADDR_ROTATION_MODE        0x000D    // 屏幕旋转模式

// 杂项
#define EMPTY_CHAR             38  // 空白字符

// *************************************
// 硬件相关
// *************************************
#define COL_GROUP_1 P0
#define COL_GROUP_2 P2
#define COL_GROUP_3 P4
#define ROW	        P3

// 开关
sbit SET_BUTTON = P1^1;
sbit ADD_BUTTON = P1^2;
sbit DIR_SENSOR = P1^3;

// 附件
sbit BUZZER = P5^0;



const uchar digitsMap[][8] = {
	{0xF1,0xEE,0xEC,0xEA,0xE6,0xEE,0xF1,0xFF},    // 0    0
	{0xFB,0xF3,0xFB,0xFB,0xFB,0xFB,0xF1,0xFF},    // 1    1
	{0xF1,0xEE,0xFE,0xFD,0xFB,0xF7,0xE0,0xFF},    // 2    2
	{0xE0,0xFD,0xFB,0xFD,0xFE,0xEE,0xF1,0xFF},    // 3    3
	{0xFD,0xF9,0xF5,0xED,0xE0,0xFD,0xFD,0xFF},    // 4    4
	{0xE0,0xEF,0xE1,0xFE,0xFE,0xEE,0xF1,0xFF},    // 5    5
	{0xF9,0xF7,0xEF,0xE1,0xEE,0xEE,0xF1,0xFF},    // 6    6
	{0xE0,0xFE,0xFD,0xFB,0xF7,0xF7,0xF7,0xFF},    // 7    7
	{0xF1,0xEE,0xEE,0xF1,0xEE,0xEE,0xF1,0xFF},    // 8    8
	{0xF1,0xEE,0xEE,0xF0,0xFE,0xFD,0xF3,0xFF},    // 9    9
	{0xF1,0xEE,0xEE,0xEE,0xE0,0xEE,0xEE,0xFF},    // A    10
	{0xE1,0xEE,0xEE,0xE1,0xEE,0xEE,0xE1,0xFF},    // B    11
	{0xF1,0xEE,0xEF,0xEF,0xEF,0xEE,0xF1,0xFF},    // C    12
	{0xE3,0xED,0xEE,0xEE,0xEE,0xED,0xE3,0xFF},    // D    13
	{0xE0,0xEF,0xEF,0xE1,0xEF,0xEF,0xE0,0xFF},    // E    14
	{0xE0,0xEF,0xEF,0xE3,0xEF,0xEF,0xEF,0xFF},    // F    15
	{0xF1,0xEE,0xEF,0xEF,0xEC,0xEE,0xF1,0xFF},    // G    16
	{0xEE,0xEE,0xEE,0xE0,0xEE,0xEE,0xEE,0xFF},    // H    17
	{0xF1,0xFB,0xFB,0xFB,0xFB,0xFB,0xF1,0xFF},    // I    18
	{0xF8,0xFD,0xFD,0xFD,0xFD,0xF5,0xF9,0xFF},    // J    19
	{0xEE,0xED,0xEB,0xE7,0xEB,0xED,0xEE,0xFF},    // K    20
	{0xEF,0xEF,0xEF,0xEF,0xEF,0xEF,0xE0,0xFF},    // L    21
	{0xEE,0xE4,0xEA,0xEE,0xEE,0xEE,0xEE,0xFF},    // M    22
	{0xEE,0xEE,0xE6,0xEA,0xEC,0xEE,0xEE,0xFF},    // N    23
	{0xF1,0xEE,0xEE,0xEE,0xEE,0xEE,0xF1,0xFF},    // O    24
	{0xE1,0xEE,0xEE,0xE1,0xEF,0xEF,0xEF,0xFF},    // P    25
	{0xF1,0xEE,0xEE,0xEE,0xEA,0xED,0xF2,0xFF},    // Q    26
	{0xE1,0xEE,0xEE,0xE0,0xEB,0xED,0xEE,0xFF},    // R    27
	{0xF0,0xEF,0xEF,0xF1,0xFE,0xFE,0xE1,0xFF},    // S    28
	{0xE0,0xFB,0xFB,0xFB,0xFB,0xFB,0xFB,0xFF},    // T    29
	{0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xF1,0xFF},    // U    30
	{0xEE,0xEE,0xEE,0xEE,0xEE,0xF5,0xFB,0xFF},    // V    31
	{0xEE,0xEE,0xEE,0xEE,0xEA,0xE4,0xEE,0xFF},    // W    32
	{0xEE,0xEE,0xF5,0xFB,0xF5,0xEE,0xEE,0xFF},    // X    33
	{0xEE,0xEE,0xF5,0xFB,0xFB,0xFB,0xFB,0xFF},    // Y    34
	{0xE0,0xFE,0xFD,0xFB,0xF7,0xEF,0xE0,0xFF},    // Z    35
	{0xFF,0xFF,0xFF,0xE1,0xFF,0xFF,0xFF,0xFF},    // -    36
	{0xE7,0xE7,0xF8,0xF7,0xF7,0xF7,0xF8,0xFF},    // ℃   37
	{0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},    //<EMP> 38
};

const uchar code pointMap[][8] = {
	{0x0F,0x0F,0x0F,0x0F,0x0F,0x0F,0x0F,0x0F},
	{0x0F,0x0F,0x0F,0x0F,0x09,0x09,0x0F,0x0F},
	{0x0F,0x09,0x09,0x0F,0x0F,0x0F,0x0F,0x0F},
	{0x0F,0x09,0x09,0x0F,0x09,0x09,0x0F,0x0F},
};

// 显示区缓存
ulong displayBuffer[] = {0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000,0x0000};

// 程序运行时间，约49天后溢出归零
ulong mills = 0;
// 显示字符缓存
uchar c1 = 1, c2 = 2, c3 = 3, c4 = 4, pointState = 3, bottomLineState = 0;
// 时间
int timeHour, timeMinute, timeSecond, tempTimeHour, tempTimeMinute;
// 显示温度时长，和不显示温度的时长。单位秒
uchar tempShowTime = 2, tempHideTime = 10;  
// 是否整点报时
uchar isRingOnTime = 1;
// 整点报时开始时间和结束时间
uchar ringOnTimeStart = 8; ringOnTimeStop = 22;
// 蜂鸣器相关
ulong buzzing = 0, startBuzzTime = 0, buzzTimeLong = 0;
// 最后一次整点报时时间
uchar lastBuzzTimeHour = 0;
// 闹铃相关
uchar isAlarm = 0, alarmHour = 0, alarmMinute = 0;
// 光感
uint brightness = 5; 
// 字符闪烁控制
bit blinkControl = 0;
// 闹铃时响铃风格相关
uchar alarming = 0, alarmBeepCount = 0, alarmed = 0; uint alarmTimestamp = 0;
// 方向
uchar rotationMode = ROTATION_MODE_AUTO, currentDirection = 0, lastDirection = 0;
ulong lastDirBlockedTime = 0;

void readSettings();
void saveSettings();
void digitsToDislpay();
void startTimer2();
void refreshDislpay();
void delayForDislpay();
void delayForSetBrightness();

void main(void)
{
    uchar i2cdata[3] = {0};
	
	ulong passedTime = 0;
	// 时间显示相关
	ulong lastBlinkTime = 0;
	// 屏幕显示控制相关
	// 0：时间；1：秒；2：小时设置；3：分钟设置
	uchar currentDisplay = MODE_SHOW_TIME; 
	// 最后一次更新屏幕显示内容的时间
	ulong lastDisplayChangeTime = 0;
	// 按钮相关
	uchar lastSetButtonState = 1, lastAddButtonState = 1, buttonState;
	// 临时存储温度
	int tempTempValue = 0;
	// 时间戳
	ulong lastSetButtonBlockedTime = 0, lastAddButtonBlockedTime = 0;
	ulong lastActionTime = 0;
    
    P0M0 = 0;
    P0M1 = 0;
    P1M0 = 0;
    P1M1 = 0;
    P2M0 = 0;
    P2M1 = 0;
    P3M0 = 0;
    P3M1 = 0;
    P4M0 = 0;
    P4M1 = 0;
	P5M0 = 0;
    P5M1 = 0;
	
	// 查看是否是安全模式
	if(SET_BUTTON == 0 && ADD_BUTTON == 0)
	{
		// 安全模式下，擦除全部EEPROM信息
		eraseEEPROM(EEP_ADDR_SECTION);
		BUZZER = 1;
		Delay1s();
		// 防止触发按钮事件
        lastSetButtonState = 0;
        lastAddButtonState = 0;
	}
	else
	{
		lastSetButtonState = SET_BUTTON;
		lastAddButtonState = ADD_BUTTON;
	}
	
	I2C_Init();	                    // 初始化I2C通讯
	startTimer2();                  // 启动Timer2(系统运行计时器)
	readSettings();
	
	DIR_SENSOR = 1;
	Delay10us();
	currentDirection = DIR_SENSOR;
	lastDirection = currentDirection;
	
	while(1)
	{
        
		// 控制闪烁，用于分割符和调整时间时的闪烁显示
		passedTime = mills - lastBlinkTime;
		if(passedTime > 500 || mills < lastBlinkTime )
		{
			blinkControl = ~blinkControl;
			lastBlinkTime = mills;
		}
	
		
		buttonState = DIR_SENSOR;
		if(buttonState != lastDirection)
		{
			passedTime = mills - lastDirBlockedTime;
			if(passedTime > ROTATE_DELAY)
			{
				currentDirection = buttonState;
				lastDirection = buttonState;
				lastDirBlockedTime = mills;
			}
		}
		else{
			lastDirBlockedTime = mills;
		}
		
		// 设置按钮动作监听
		buttonState = SET_BUTTON;
		if(lastSetButtonState != buttonState)
		{
			// 每个动作必须超过50毫秒才会被承认
			passedTime = mills - lastSetButtonBlockedTime;
			if(passedTime > 50)
			{
				lastSetButtonState = buttonState;
				lastSetButtonBlockedTime = mills;
				if(buttonState == 0)
				{
					if(alarming)
					{
						alarming = 0;
					
						BUZZER = 0;
						alarmed = 1;
						continue;
					}
					// 如果当前显示时间
					if(currentDisplay == MODE_SHOW_TIME || currentDisplay == MODE_SHOW_TEMPERTURE)
					{
						currentDisplay = MODE_SET_HOUR;
						tempTimeHour = timeHour;
						tempTimeMinute = timeMinute;
						lastActionTime = mills;
					}
					// 如果当前显示秒
					else if(currentDisplay == MODE_SHOW_SECONDS)
					{
						// 直接将秒数设置为0
						I2C_Put(0xd0, 0x00, 0x00); // sec
					}
					else if(currentDisplay == MODE_SET_HOUR)
					{
						currentDisplay = MODE_SET_MINUTE;
						lastActionTime = mills;
					}
					else if(currentDisplay == MODE_SET_MINUTE)
					{
						lastDisplayChangeTime = mills;
						lastActionTime = mills;
						
						// 防止出现设置成整数时间直接整点报时的行为
						lastBuzzTimeHour = tempTimeHour;
						
						// 与DS3231通讯设置时间
						I2C_Put(0xd0, 0x01, tempTimeMinute + 6 * (tempTimeMinute / 10)); // min
						I2C_Put(0xd0, 0x02, tempTimeHour + 6 * (tempTimeHour / 10)); // hour
						
						currentDisplay = MODE_SET_BRIGHTNESS;
					}
					else if(currentDisplay == MODE_SET_BRIGHTNESS)
					{
						saveSettings();
						currentDisplay = MODE_SHOW_TIME;
					}
				}
			}
		}
		else
		{
			lastSetButtonBlockedTime = mills;
		}
		
		// 调整按钮动作监听
		buttonState = ADD_BUTTON;
		if(lastAddButtonState != buttonState)
		{
			// 每个动作必须超过50毫秒才会被承认
			passedTime = mills - lastAddButtonBlockedTime;
			if(passedTime > 50)
			{
				lastAddButtonState = buttonState;
				lastAddButtonBlockedTime = mills;
				
				if(alarming)
				{
					alarming = 0;
				
					BUZZER = 0;
					alarmed = 1;
					continue;
				}
					
				// 如果按钮状态为按下
				if(buttonState == 0)
				{
					// 如果当前显示时间
					if(currentDisplay == MODE_SHOW_TIME || currentDisplay == MODE_SHOW_TEMPERTURE)
					{
						currentDisplay = MODE_SHOW_SECONDS;
					}
					// 如果当前显示秒
					else if(currentDisplay == MODE_SHOW_SECONDS)
					{
						currentDisplay = MODE_SHOW_TIME;
						lastDisplayChangeTime = mills;
					}
					// 如果当前正在调整时间（小时）
					else if(currentDisplay == MODE_SET_HOUR)
					{
						tempTimeHour ++;
						tempTimeHour = tempTimeHour >= 24 ? 0 : tempTimeHour;
						
						lastBlinkTime = mills;
						lastActionTime = mills;
						blinkControl = 0;
					}
					// 如果当前正在调整时间（分钟）
					else if(currentDisplay == MODE_SET_MINUTE)
					{
						tempTimeMinute ++;
						tempTimeMinute = tempTimeMinute >= 60 ? 0 : tempTimeMinute;
						
						lastBlinkTime = mills;
						lastActionTime = mills;
						blinkControl = 0;
					}
					// 如果当前正在调整是否显示闹铃
					else if(currentDisplay == MODE_SET_ALARM_ENABLED)
					{
						isAlarm = isAlarm ? 0 : 1;
						
						lastBlinkTime = mills;
						lastActionTime = mills;
						blinkControl = 0;
					}
					// 如果当前正在调整闹铃时
					else if(currentDisplay == MODE_SET_ALARM_HOUR)
					{
						alarmHour ++;
						alarmHour = alarmHour >= 24 ? 0 : alarmHour;
						
						lastBlinkTime = mills;
						lastActionTime = mills;
						blinkControl = 0;
					}
					// 如果当前正在调整闹铃分
					else if(currentDisplay == MODE_SET_ALARM_MINUTE)
					{
						alarmMinute ++;
						alarmMinute = alarmMinute >= 60 ? 0 : alarmMinute;
						
						lastBlinkTime = mills;
						lastActionTime = mills;
						blinkControl = 0;
					}
					// 如果当前正在调整温度显示时间
					else if(currentDisplay == MODE_SET_TEMP_SHOW)
					{
						tempShowTime ++;
						tempShowTime = tempShowTime > TEMP_MAX_SHOW_TIME ? 0 : tempShowTime;
						
						if(tempShowTime == 0 && tempHideTime == 0)
						{
							tempHideTime = 1;
						}
						
						lastBlinkTime = mills;
						lastActionTime = mills;
						blinkControl = 0;
					}
					// 如果当前正在调整温度隐藏时间
					else if(currentDisplay == MODE_SET_TEMP_NO_SHOW)
					{
						tempHideTime ++;
						tempHideTime = tempHideTime > TEMP_MAX_HIDE_TIME ? 0 : tempHideTime;
						if(tempShowTime == 0 && tempHideTime == 0)
						{
							tempHideTime = 1;
						}
						
						lastBlinkTime = mills;
						lastActionTime = mills;
						blinkControl = 0;
					}
					// 如果当前正在设置是否整点报时
					else if(currentDisplay == MODE_SET_ROT_ENABLED)
					{
						isRingOnTime = isRingOnTime ? 0 : 1;
						
						lastBlinkTime = mills;
						lastActionTime = mills;
						blinkControl = 0;
					}
					// 如果当前正在设置整点报时启动时间
					else if(currentDisplay == MODE_SET_ROT_START)
					{
						ringOnTimeStart ++;
						ringOnTimeStart = ringOnTimeStart >= 24 ? 0 : ringOnTimeStart;
						if(ringOnTimeStop < ringOnTimeStart)
						{
							ringOnTimeStop = ringOnTimeStart;
						}
						
						lastBlinkTime = mills;
						lastActionTime = mills;
						blinkControl = 0;
					}
					// 如果当前正在设置整点报时停止时间
					else if(currentDisplay == MODE_SET_ROT_END)
					{
						ringOnTimeStop ++;
						ringOnTimeStop = ringOnTimeStop >= 24 ? 0 : ringOnTimeStop;
						
						if(ringOnTimeStop < ringOnTimeStart)
						{
							ringOnTimeStop = ringOnTimeStart;
						}
						
						lastBlinkTime = mills;
						lastActionTime = mills;
						blinkControl = 0;
					}
					else if(currentDisplay == MODE_SET_BRIGHTNESS)
					{
						brightness ++;
						brightness = brightness > MAX_BRIGHTNESS ? 0 : brightness;
						
						lastBlinkTime = mills;
						lastActionTime = mills;
						blinkControl = 0;
					}
				}
			}
		}
		else
		{
			lastAddButtonBlockedTime = mills;
		}
		
		// 进入设置时间后长时间无操作自动退出
		if(currentDisplay == MODE_SET_HOUR 
			|| currentDisplay == MODE_SET_MINUTE
			|| currentDisplay == MODE_SET_ALARM_ENABLED
			|| currentDisplay ==  MODE_SET_ALARM_HOUR
			|| currentDisplay ==  MODE_SET_ALARM_MINUTE
			|| currentDisplay ==  MODE_SET_TEMP_SHOW
			|| currentDisplay ==  MODE_SET_TEMP_NO_SHOW
			|| currentDisplay ==  MODE_SET_ROT_ENABLED
			|| currentDisplay ==  MODE_SET_ROT_END
			|| currentDisplay ==  MODE_SET_ROT_START)
		{
			passedTime = mills - lastActionTime;
			if(passedTime >= TIME_SET_TIMEOUT)
			{
				currentDisplay = MODE_SHOW_TIME;
				lastDisplayChangeTime = mills;
			}
		}
		
		// 如果显示内容为时间
		if(currentDisplay == MODE_SHOW_TIME)
		{
			// I2C_Get(0xd0,0x02,i2cdata);
			// timeHour = ((i2cdata[0]>>4) & 0x03)*10 + (i2cdata[0]&0x0F);
			
			// I2C_Get(0xd0,0x01,i2cdata);
			// timeMinute = (i2cdata[0]>>4)*10 + (i2cdata[0]&0x0F);
			
			// I2C_Get(0xd0,0x00,i2cdata);
			// timeSecond = (i2cdata[0]>>4)*10 + (i2cdata[0]&0x0F);
			
			I2C_Gets(0xd0, 0x00, 3, i2cdata);
			
			timeSecond = (i2cdata[0]>>4)*10 + (i2cdata[0]&0x0F);
			timeMinute = (i2cdata[1]>>4)*10 + (i2cdata[1]&0x0F);
			timeHour = ((i2cdata[2]>>4) & 0x03)*10 + (i2cdata[2]&0x0F);
			
			c1 = timeHour / 10;
			c2 = timeHour % 10;
			c3 = timeMinute / 10;
			c4 = timeMinute % 10;

			pointState = blinkControl ? 0 : 3;   
			bottomLineState = ((int)(timeSecond / 2.5));
		}
		// 如果显示的内容为秒
		else if(currentDisplay == MODE_SHOW_SECONDS)
		{
			I2C_Gets(0xd0, 0x00, 3, i2cdata);
			
			timeSecond = (i2cdata[0]>>4)*10 + (i2cdata[0]&0x0F);
			timeMinute = (i2cdata[1]>>4)*10 + (i2cdata[1]&0x0F);
			timeHour = ((i2cdata[2]>>4) & 0x03)*10 + (i2cdata[2]&0x0F);
			
			c1 = EMPTY_CHAR;
			c2 = EMPTY_CHAR;
			c3 = timeSecond / 10;
			c4 = timeSecond % 10;
			
			pointState = 3;
			bottomLineState = ((int)(timeSecond / 2.5));
		}
		// 如果显示内容为小时调节
		else if(currentDisplay == MODE_SET_HOUR)
		{
			c1 = blinkControl ? EMPTY_CHAR : (tempTimeHour / 10);
			c2 = blinkControl ? EMPTY_CHAR : (tempTimeHour % 10);
			c3 = tempTimeMinute / 10;
			c4 = tempTimeMinute % 10;
			
			pointState = 3;
			bottomLineState = 0;
		}
		// 如果显示内容为分钟调节
		else if(currentDisplay == MODE_SET_MINUTE)
		{
			c1 = tempTimeHour / 10;
			c2 = tempTimeHour % 10;
			c3 = blinkControl ? EMPTY_CHAR : (tempTimeMinute / 10);
			c4 = blinkControl ? EMPTY_CHAR : (tempTimeMinute % 10);
			
			pointState = 3;
			bottomLineState = 0;
		}
		// 如果显示内容为分钟调节
		else if(currentDisplay == MODE_SET_BRIGHTNESS)
		{
			c1 = 11;
			c2 = 27;
			c3 = blinkControl ? EMPTY_CHAR : brightness / 10;
			c4 = blinkControl ? EMPTY_CHAR : brightness % 10;
			
			pointState = 3;
			bottomLineState = 0;
			
			delayForSetBrightness();
		}
		
		
		digitsToDislpay();
		refreshDislpay();
	}

}

/**
 * 定时器2初始化
 * 定时器2间隔1毫秒tick一次，用于提供一个基准来获取系统运行时长
 */
void startTimer2(void)
{
    AUXR |= 0x04;		//定时器时钟1T模式
	T2L   = 0x9A;		//设置定时初值
	T2H   = 0xA9;		//设置定时初值
	AUXR |= 0x10;		//定时器2开始计时
	IE2  |= 0x04;
	EA = 1;
}

void timer2Tick() interrupt 12
{
    mills ++;
}

void digitsToDislpay()
{
	int i;
	ulong buf;
	for(i = 0; i < 7; i++)
	{
		displayBuffer[i] = 0x0000;
	}
	for(i = 0; i < 7; i++)
	{
		displayBuffer[i] |= digitsMap[c1][i] & 0x1F;
		displayBuffer[i] = displayBuffer[i] << 5 ;
	}
	
	
	for(i = 0; i < 7; i++)
	{
		displayBuffer[i] |=  digitsMap[c2][i] & 0x1F;
		displayBuffer[i] = displayBuffer[i] << 4;
	}
	
	for(i = 0; i < 7; i++)
	{
		displayBuffer[i] |=  pointMap[pointState][i];
		displayBuffer[i] = displayBuffer[i] << 5;
	}
	
	for(i = 0; i < 7; i++)
	{
		displayBuffer[i] |=  digitsMap[c3][i] & 0x1F;
		displayBuffer[i] = displayBuffer[i] << 5;
	}
	
	for(i = 0; i < 7; i++)
	{
		displayBuffer[i] |=  digitsMap[c4][i] & 0x1F;
	}
	
	displayBuffer[7] = 0x00FFFFFFL >> bottomLineState;
	
	// 如果是倒置，旋转屏幕
	if(rotationMode == ROTATION_MODE_REVERSE 
		|| (rotationMode == ROTATION_MODE_AUTO && currentDirection == 0))
	{
		// 行反转
		buf = displayBuffer[0];
		displayBuffer[0] = displayBuffer[7]; displayBuffer[7] = buf;
		buf = displayBuffer[1];
		displayBuffer[1] = displayBuffer[6]; displayBuffer[6] = buf;
		buf = displayBuffer[2];
		displayBuffer[2] = displayBuffer[5]; displayBuffer[5] = buf;
		buf = displayBuffer[3];
		displayBuffer[3] = displayBuffer[4]; displayBuffer[4] = buf;
        
        // 高低位反转
		for(i = 0; i < 8; i++)
		{
			// 蝶式反转法
			displayBuffer[i]=(displayBuffer[i]<<16)|(displayBuffer[i]>>16);  
            displayBuffer[i]=(displayBuffer[i]<<8&0xFF00FF00)|(displayBuffer[i]>>8&0x00FF00FF); 
            displayBuffer[i]=(displayBuffer[i]<<4&0xF0F0F0F0)|(displayBuffer[i]>>4&0x0F0F0F0F);  
            displayBuffer[i]=(displayBuffer[i]<<2&0xCCCCCCCC)|(displayBuffer[i]>>2&0x33333333);  
            displayBuffer[i]=(displayBuffer[i]<<1&0xAAAAAAAA)|(displayBuffer[i]>>1&0x55555555);
            displayBuffer[i] = displayBuffer[i] >> 8;
		} 
	} 
}

void refreshDislpay()
{
	int i;
	for(i = 0; i < 8; i++)
	{
		
		ROW = 0xFF;
		COL_GROUP_1 = displayBuffer[i] >> 16;
		COL_GROUP_2 = displayBuffer[i] >> 8;
		COL_GROUP_3 = displayBuffer[i];
		ROW = ~(0x01 << i);
		delayForDislpay();
	}
	
	ROW = 0xFF;
}

void delayForDislpay()
{
	int i;
	for(i = 0; i <= brightness; i++)
	{
		Delay100us();
	}
}

void delayForSetBrightness()
{
	Delay1ms();
	Delay1ms();
}

void readSettings(){
	uchar dat;
	
	dat = readByteFromEEROM(EEP_ADDR_VERSION);
	if(dat != 0xFF && dat != EEPROM_VERSION){
		eraseEEPROM(EEP_ADDR_SECTION);
		delay500us();
		writeByteToEEPROM(EEP_ADDR_VERSION, EEPROM_VERSION);
	    delay500us();
		saveSettings();
		return;
	}
	
	dat = readByteFromEEROM(EEP_ADDR_TEMP_VISIBLE);
	if(dat != 0xFF){
		tempShowTime = dat;
	}
	
	dat = readByteFromEEROM(EEP_ADDR_TEMP_INVISIBLE);
	if(dat != 0xFF){
		tempHideTime = dat;
	}
	
	dat = readByteFromEEROM(EEP_ADDR_IS_RING_ON_TIME);
	if(dat != 0xFF){
		isRingOnTime = dat;
	}
	
	dat = readByteFromEEROM(EEP_ADDR_RING_ON_TIME_START);
	if(dat != 0xFF){
		ringOnTimeStart = dat;
	}
	
	dat = readByteFromEEROM(EEP_ADDR_RING_ON_TIME_STOP);
	if(dat != 0xFF){
		ringOnTimeStop = dat;
	}
	
	dat = readByteFromEEROM(EEP_ADDR_ALARM_ENABLE);
	if(dat != 0xFF){
		isAlarm = dat;
	}
	
	dat = readByteFromEEROM(EEP_ADDR_ALARM_HOUR);
	if(dat != 0xFF){
		alarmHour = dat;
	}
	
	dat = readByteFromEEROM(EEP_ADDR_ALARM_MINUTE);
	if(dat != 0xFF){
		alarmMinute = dat;
	}
	
	dat = readByteFromEEROM(EEP_ADDR_BRIGHTNESS);
	if(dat != 0xFF){
		brightness = dat;
	}
}

void saveSettings(){
    uint i = 0;
	//扇区擦除
	eraseEEPROM(EEP_ADDR_SECTION);
	delay500us();
    //检测是否擦除成功(全FF检测)
	for (i=0; i<512; i++)           
    {
        if (readByteFromEEROM(EEP_ADDR_SECTION+i) != 0xff)
            return;
    }
	// 写入数据
	writeByteToEEPROM(EEP_ADDR_TEMP_VISIBLE, tempShowTime);
	delay500us();
	writeByteToEEPROM(EEP_ADDR_TEMP_INVISIBLE, tempHideTime);
	delay500us();
	writeByteToEEPROM(EEP_ADDR_IS_RING_ON_TIME, isRingOnTime);
	delay500us();
	writeByteToEEPROM(EEP_ADDR_RING_ON_TIME_START, ringOnTimeStart);
	delay500us();
	writeByteToEEPROM(EEP_ADDR_RING_ON_TIME_STOP, ringOnTimeStop);
	delay500us();
	writeByteToEEPROM(EEP_ADDR_ALARM_ENABLE, isAlarm);
	delay500us();
	writeByteToEEPROM(EEP_ADDR_ALARM_HOUR, alarmHour);
	delay500us();
	writeByteToEEPROM(EEP_ADDR_ALARM_MINUTE, alarmMinute);
	delay500us();
	writeByteToEEPROM(EEP_ADDR_BRIGHTNESS, brightness);
	delay500us();
}